// Test program for gateStep() function which is part of railwaydemov4

#include <Servo.h>     // add library functions needed for controlling servo

#define SERVO 10       // control signal pin for servo

#define GATE_OPEN 1         // value for state of gate: open
#define GATE_CLOSED 2       // value for state of gate: closed
#define GATE_OPENING 3      // value for state of gate: opening
#define GATE_CLOSING 4      // value for state of gate: closing
#define GATE_CHANGEPERIOD 40       // period for timing of closing of gate in ms
#define GATE_CLOSEDPERIOD 10000    // period the gate is closed in ms

#define SERVO_OPENPOS 90     // value for servo position when gate is open
#define SERVO_CLOSEPOS 10    // value for servo position when gate is closed
#define SERVO_STEPS 2        // number of steps made by servo per GATE_CHANGEPERIOD opening/closing

// declarations and initializations; these variables have a global scope
int gateState = GATE_OPEN;               // variable indicating whether gate is open or closed
int prevgateState = GATE_OPEN;           // value of gateState in previous pass of loop()

unsigned long  gateChangingTimer = GATE_CHANGEPERIOD;   // variable for timing the opening/closing of the gate
int servoPos = SERVO_OPENPOS;                           // servo position

Servo myServo;                           // variable instance of servo

void setup() {
  // put your setup code here, to run once:
  myServo.attach(SERVO);
  myServo.write(servoPos);            // move servo to open position
}

void loop() {
  prevgateState = gateState;
  gateState = GATE_OPEN;
  gateStep();   // do the work for opening
  
  delay(2000);
  
  prevgateState = gateState;
  gateState = GATE_CLOSING;
  while (gateState != GATE_CLOSED)    // keep calling gateStep() to close gate until it indicates it is closed
    gateStep();
  
  delay(2000);
  gateStep();
  
  prevgateState = gateState;
  gateState = GATE_OPENING;
  while (gateState != GATE_OPEN)    // keep calling gateStep() to open gate until it indicates it is open
    gateStep();
  
  delay(2000);
  
  gateStep();
}

void gateStep()
{
  switch (gateState)
  {
    case GATE_CLOSED:
      myServo.write(SERVO_CLOSEPOS);
      break;

    case GATE_OPEN:
      myServo.write(SERVO_OPENPOS);
      break;

    case GATE_CLOSING:
      if (prevgateState != gateState) { // when entering GATE_CLOSING first time
        prevgateState = gateState;
        gateChangingTimer = millis();  // initialize time for closing gate
      }
      else {
        if (millis() - gateChangingTimer >= GATE_CHANGEPERIOD) {
          servoPos = servoPos - SERVO_STEPS;   // remember SERVOPENPOS > SERVO_CLOSEPOS
          servoPos = constrain(servoPos, SERVO_CLOSEPOS, SERVO_OPENPOS);
          myServo.write(servoPos);
          gateChangingTimer = millis();  //re-initialize time for next step closing gate

          if (servoPos <= SERVO_CLOSEPOS) {  // adapt gateState when gate has been closed.
            gateState = GATE_CLOSED;
          }
        }
      }
      break;

    case GATE_OPENING:
      if (prevgateState != gateState) { // when entering GATE_OPENING first time
        prevgateState = gateState;
        gateChangingTimer = millis();  // initialize time for closing gate
      }
      else {
        if (millis() - gateChangingTimer >= GATE_CHANGEPERIOD) {
          servoPos = servoPos + SERVO_STEPS;   // remember SERVOPENPOS > SERVO_CLOSEPOS
          servoPos = constrain(servoPos, SERVO_CLOSEPOS, SERVO_OPENPOS);
          myServo.write(servoPos);
          gateChangingTimer = millis();   //re-initialize time for next step opening gate

          if (servoPos >= SERVO_OPENPOS) {  // adapt gateState when gate has been opened
            gateState = GATE_OPEN;
          }
        }
      }
      break;
  }
}

